@@ -233,8 +233,10 @@ plot.demonoid.ppc <- function(x, Style=NULL, Data=NULL, Rows=NULL,
           temp <- summary(x, Quiet=TRUE)$Summary
           for (s in 1:Data$S) {
                plot(matrix(temp[,1], Data$S, Data$T)[s,],
-                    ylim=c(min(matrix(temp[,4], Data$S, Data$T)[s,], na.rm=TRUE),
-                         max(matrix(temp[,6], Data$S, Data$T)[s,], na.rm=TRUE)),
+                    ylim=c(min(c(matrix(temp[,4], Data$S, Data$T)[s,],
+                         matrix(temp[,1], Data$S, Data$T)[s,]), na.rm=TRUE),
+                         max(c(matrix(temp[,6], Data$S, Data$T)[s,],
+                         matrix(temp[,1], Data$S, Data$T)[s,]), na.rm=TRUE)),
                     type="l", xlab="Time", ylab="y",
                     main=paste("Space-Time at Space s=",s," of ",
                          Data$S, sep=""),
